/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2007
 *
 *    File name   : main.c
 *    Description : Main module
 *
 *    History :
 *    1. Date        : 20, December 2007
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *   This example project shows how to use the IAR Embedded Workbench
 *  for ARM to develop code for the Olimex STM32-103STK board.
 *   It implements wireless point to point USB CDC (Communication Device Class)
 *  device and install it like a Virtual COM port.
 *   The connection realize between device with Node A configuration and
 *  Node B configuration.
 *
 *    $Revision: 1.1.2.1 $
 **************************************************************************/
#include "includes.h"

#define LED_ON()          GPIO_WriteBit(GPIOC,GPIO_Pin_12,Bit_RESET)
#define LED_OFF()         GPIO_WriteBit(GPIOC,GPIO_Pin_12,Bit_SET)

#define RING_BUFFER_SIZE  150

Int32U CriticalSecCntr;

RingBuffer_t RfRxRingBuffer;
RingBuffer_t RfTxRingBuffer;
Int8U RxBuff[RING_BUFFER_SIZE];
Int8U TxBuff[RING_BUFFER_SIZE];

extern void RfInit(void);
extern void StartRx(void);
/*************************************************************************
 * Function Name: Clk_Init
 * Parameters: Int32U Frequency
 * Return: Int32U
 *
 * Description: Init clock system
 *
 *************************************************************************/
void Clk_Init (void)
{
  // 1. Clocking the controller from internal HSI RC (8 MHz)
  RCC_HSICmd(ENABLE);
  // wait until the HSI is ready
  while(RCC_GetFlagStatus(RCC_FLAG_HSIRDY) == RESET);
  RCC_SYSCLKConfig(RCC_SYSCLKSource_HSI);
  // 2. Enable ext. high frequency OSC
  RCC_HSEConfig(RCC_HSE_ON);
  // wait until the HSE is ready
  while(RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET);
  // 3. Init PLL
  RCC_PLLConfig(RCC_PLLSource_HSE_Div1,RCC_PLLMul_9); // 72MHz
  RCC_PLLCmd(ENABLE);
  // wait until the PLL is ready
  while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);
  // 4. Set system clock dividers
  RCC_USBCLKConfig(RCC_USBCLKSource_PLLCLK_1Div5);
  RCC_ADCCLKConfig(RCC_PCLK2_Div8);
  RCC_PCLK2Config(RCC_HCLK_Div1);
  RCC_PCLK1Config(RCC_HCLK_Div2);
  RCC_HCLKConfig(RCC_SYSCLK_Div1);
#ifdef EMB_FLASH
  // 5. Init Embedded Flash
  // Zero wait state, if 0 < HCLK 24 MHz
  // One wait state, if 24 MHz < HCLK 56 MHz
  // Two wait states, if 56 MHz < HCLK 72 MHz
  // Flash wait state
  FLASH_SetLatency(FLASH_Latency_2);
  // Half cycle access
  FLASH_HalfCycleAccessCmd(FLASH_HalfCycleAccess_Disable);
  // Prefetch buffer
  FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
#endif // EMB_FLASH
  // 5. Clock system from PLL
  RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
void main(void)
{
Int32U Size;
GPIO_InitTypeDef GPIO_InitStructure;
pInt8U pBuffer;
Boolean bComm;

#ifdef DEBUG
 debug();
#endif

  ENTR_CRT_SECTION();

  // Init clock system
  Clk_Init();

  // Init LED Output
  RCC_APB2PeriphResetCmd(  RCC_APB2Periph_GPIOC, DISABLE);
  RCC_APB2PeriphClockCmd(  RCC_APB2Periph_GPIOC, ENABLE);

  // Configure PC12 STAT LED & LCD LE PIN.
  GPIO_WriteBit(GPIOC,GPIO_Pin_12,Bit_SET);

  GPIO_WriteBit(GPIOC,GPIO_Pin_10,Bit_SET);

  GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_12 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOC, &GPIO_InitStructure);

  // NVIC init
#ifndef  EMB_FLASH
  /* Set the Vector Table base location at 0x20000000 */
  NVIC_SetVectorTable(NVIC_VectTab_RAM, 0x0);
#else  /* VECT_TAB_FLASH  */
  /* Set the Vector Table base location at 0x08000000 */
  NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x0);
#endif
  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);

  // Enable timer counting
  TIM1_Cmd(ENABLE);

  // CDC USB
  UsbCdcInit();

  // Soft connection enable
  USB_ConnectRes(TRUE);

  // Init Ring Buffers
  RingBuff_Init(&RfRxRingBuffer,RxBuff,RING_BUFFER_SIZE);
  RingBuff_Init(&RfTxRingBuffer,TxBuff,RING_BUFFER_SIZE);

  RfInit();

  EXT_CRT_SECTION();

  while(1)
  {
    if (IsUsbCdcConfigure())
    {
      bComm = FALSE;
      // Get free size of the buffer
      pBuffer = RingBuff_GetFreeLinBuff(&RfTxRingBuffer,&Size);
      if(Size)
      {
        // Read data from USB
        Size = UsbCdcRead(pBuffer,Size);
        if(Size)
        {
          LED_ON();
          bComm = TRUE;
        }
        RingBuff_Allocate(&RfTxRingBuffer,Size);
      }

      // Get filled size of the buffer
      pBuffer = RingBuff_GetUsedLinBuff(&RfRxRingBuffer,&Size);
      if(Size)
      {
        // Send data from radio
        LED_ON();
        bComm = TRUE;
        UsbCdcWrite(pBuffer,Size);
        RingBuff_Free(&RfRxRingBuffer,Size);
      }

      if (!bComm)
      {
        LED_OFF();
      }
    }
    else
    {
      LED_OFF();
    }
  }
}

#ifdef  DEBUG
/*******************************************************************************
* Function Name  : assert_failed
* Description    : Reports the name of the source file and the source line number
*                  where the assert_param error has occurred.
* Input          : - file: pointer to the source file name
*                  - line: assert_param error line source number
* Output         : None
* Return         : None
*******************************************************************************/
void assert_failed(u8* file, u32 line)
{
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif